@@ -1,7 +1,7 @@
 const {eq} = require("prosemirror-test-builder")
 const ist = require("ist")
 
-const {schema, defaultMarkdownParser, defaultMarkdownSerializer} = require("..")
+const {schema, defaultMarkdownParser, defaultMarkdownSerializer, MarkdownSerializer} = require("..")
 
 const {doc, blockquote, h1, h2, p, hr, li, ol, ol3, ul, pre, em, strong, code, a, link, br, img} = require("./build")
 
@@ -95,6 +95,16 @@ describe("markdown", () => {
          doc(p(link({href: "foo.html"}, "foo.html"))))
   })
 
+  it("can handle link titles", () => {
+    same('[a](x.html "title \\"quoted\\"")',
+         doc(p(link({href: "x.html", title: 'title "quoted"'}, "a"))))
+  })
+
+  it("doesn't escape underscores in link", () => {
+    same('[link](http://foo.com/a_b_c)',
+         doc(p(link({href: "http://foo.com/a_b_c"}, "link"))))
+  })
+
   it("parses emphasized urls", () =>
      same("Link to *<https://prosemirror.net>*",
           doc(p("Link to ", em(link({href: "https://prosemirror.net"}, "https://prosemirror.net"))))))
@@ -144,4 +154,46 @@ describe("markdown", () => {
 
   it("doesn't escape characters in code", () =>
      same("foo`*`", doc(p("foo", code("*")))))
+
+  it("doesn't escape underscores between word characters", () =>
+     same(
+       "abc_def",
+       doc(p("abc_def"))
+     )
+   )
+
+   it("doesn't escape strips of underscores between word characters", () =>
+     same(
+       "abc___def",
+       doc(p("abc___def"))
+     )
+   )
+
+   it("escapes underscores at word boundaries", () =>
+     same(
+       "\\_abc\\_",
+       doc(p("_abc_"))
+     )
+   )
+
+   it("escapes underscores surrounded by non-word characters", () =>
+     same(
+       "/\\_abc\\_)",
+       doc(p("/_abc_)"))
+     )
+   )
+
+  context("custom serializer", () => {
+   let markdownSerializer = new MarkdownSerializer(
+     defaultMarkdownSerializer.nodes,
+     defaultMarkdownSerializer.marks,
+     {
+       escapeExtraCharacters: /[\|!]/g,
+     }
+   );
+
+   it("escapes extra characters from options", () => {
+     ist(markdownSerializer.serialize(doc(p("foo|bar!"))), "foo\\|bar\\!");
+   });
+ });
 })
